#!/usr/bin/bash

# 
# File: player.sh
# Version: 0.2
# Authors: Stéphane Cousot <http://www.ubaa.net/>
# License: player.sh is released under GNU General Public License, version 3 or later
#
# --------------------------------------------------------------------------------------
#
# This file is part of the Locus Sonus Streambox Player project.
# http://locusonus.org/streambox/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

source /boot/settings.txt



# TODO: trim PLAYLIST

IFS=';' read -r -a LIST <<< "$PLAYLIST"
LEN=${#LIST[*]}


play()
{
	stop

	SRC=${LIST[$1]// /\\ }

	echo "-- play [$1]: ${LIST[$1]}"
	echo "-- command: $COMMAND_LINE $SRC"
	echo
	$COMMAND_LINE "${LIST[$1]}"
}


stop()
{
	IFS=' ' read -r -a CMD <<< "$COMMAND_LINE"
	PID=`pidof ${CMD[0]}`

	if [ ! -z "$PID" ];then
		echo "-- kill (PID: $PID): ${CMD[0]}"
		kill -s SIGINT $PID
	fi
}

waitfor()
{
	IDX=$1
	URL=$2

	echo -e "\033[1;35m>> check stream [$IDX] : $URL \033[0m"
	CODE=`curl -s "http://locusonus.org/soundmap/checkurl?q=$URL"`
	
	if [[ "$CODE" && $CODE -ne 200 ]]; then
		echo -e "\033[1;35m>> waiting for stream [$IDX] to be online ...\033[0m"
		while [[ "$CODE" && $CODE -ne 200 ]]
		do
			sleep 7
			CODE=`curl -s "http://locusonus.org/soundmap/checkurl?q=$URL"`
		done
	fi


	if [[ "$CODE" && $CODE -eq 200 ]]; then
		echo -e "\033[1;35m>> stream [$IDX] is online\033[0m"
		if [ $(</dev/shm/pl_index) -ge $IDX ]; then
			echo -e "\033[1;35m>> revert to playlist item [$IDX] : $URL \033[0m"
			echo $((IDX-1)) >/dev/shm/pl_index
			stop
		fi
	fi
	
	echo 0 >/dev/shm/pl_waitfor-$IDX
	echo -e "\033[1;35m>> close the check process for stream [$IDX] \033[0m"
	exit 0
}



end()
{
	RUNNING=0
	echo "-- stop running..."
	stop
}

main()
{

	echo "-- playlist length: $LEN"


	# to reduce disk load, use shared memory /dev/shm to store variable
	# current playlist index
	echo 0 >/dev/shm/pl_index
	# mark if async process is already create for playlist index
	for (( i=0; i<$LEN; ++i )); do
		echo 0 >/dev/shm/pl_waitfor-$i
	done

	# run
	while [ $RUNNING -eq 1 ]
	do

		IDX=$(</dev/shm/pl_index)
		play $IDX

		# update
		if [ $RUNNING -eq 1 ]; then

			STREAM=`echo ${LIST[$IDX]// /\\ } | grep -e "http://"`

			# wait for stream to be online: run subprocess async
			if [[ $STREAM && `cat /dev/shm/pl_waitfor-$IDX` -eq 0 ]]; then
				waitfor $IDX ${LIST[$IDX]} &
				echo $! >/dev/shm/pl_waitfor-$IDX
			fi

			# increment curent index
			echo $(($(</dev/shm/pl_index)+1)) >/dev/shm/pl_index;
			 # loop playlist
			if [ $(</dev/shm/pl_index) -ge $LEN ]; then echo 0 >/dev/shm/pl_index; fi
		fi

	done
}

trap end SIGINT

RUNNING=1
main

exit $?

