/**
 * (./) streambox-player.js
 * (by) cousot stéphane @ http://www.ubaa.net/
 * (cc) some right reserved
 *
 *
 * User interaction, request/response dispatcher, frontend process.
 * version 0.1
 *
 * ---------------------------------------------------------------------------------------------------------------------
 *
 *  This file is part of the Locus Sonus Streambox Player project. http://locusonus.org/streambox/
 *
 *  THIS CODE ARE RELEASED UNDER A CREATIVE COMMONS ATTRIBUTION 4.0 INTERNATIONAL LICENSE
 *  @ http://creativecommons.org/licenses/by/4.0/
 *
 * ---------------------------------------------------------------------------------------------------------------------
 */


const FETCH_STATUS_INTERVAL = 60*1000;
const AJAX_TIMEOUT_DELAY 	= 20*1000;
const AJAX_PING_FREQ 		= 7*1000;
const PANEL_TOGGLE_DURATION = 400;
const FADEOUT_OK_DELAY   	= 1600;
const FADEOUT_ERR_DELAY  	= 6000;

const CGI = "php/process.php";
const AUDIO_CGI = "/home/locusonus/bin/audio.sh";
const NETWORK_CGI = "/home/locusonus/bin/network.sh";



$(document).ready(function()
{
	// stop propagation 
	$("a[href='#']").click(function(event){ event.preventDefault(); });

	// add toggle panel
	$("h2").click(function(e)
	{
		var $elem = $(this);

		if ( $elem.data('onopen') && $elem.siblings(".subpanel").is(":hidden") ) 
			eval( $elem.data('onopen')+'();' );
		if ( $elem.data('onclose') && !$elem.siblings(".subpanel").is(":hidden") ) 
			eval( $elem.data('onclose')+'();' ); 

		$(this).siblings(".subpanel").slideToggle( PANEL_TOGGLE_DURATION, function()
		{
			if ( $elem.data('oncomplete') && !$(this).is(":hidden") )
				eval( $elem.data('oncomplete')+'();' ); 
		});
		$(this).toggleClass("open");
	})
	.addClass('switch');

	$( ".subpanel" ).prev(":header").click(function(e)
	{
		var $elem = $(this);

		if ( $elem.data('onopen') && $elem.next(".subpanel").is(":hidden") ) 
			eval( $elem.data('onopen')+'();' );
		if ( $elem.data('onclose') && !$elem.next(".subpanel").is(":hidden") ) 
			eval( $elem.data('onclose')+'();' ); 

		$(this).siblings(".subpanel").hide(PANEL_TOGGLE_DURATION);
		$(this).next(".subpanel").stop().slideToggle( PANEL_TOGGLE_DURATION, function()
		{
			if ( $elem.data('oncomplete') && !$(this).is(":hidden") )
				eval( $elem.data('oncomplete')+'();' ); 
		});
		$(this).toggleClass("open");
	})
	.addClass('switch');


	// // editable input
	// $("input.editable").each(function()
	// {
	// 	$('<SPAN/>').dblclick(function()
	// 	{
	// 		$(this).hide();
	// 		$('#'+$(this).data('id')).val( $(this).text() ).show();
	// 	})
	// 	.data('id', $(this).attr('id') )
	// 	.addClass( $(this).attr('class') ).addClass('input')
	// 	.css('font-family',$('input').css('font-family'))
	// 	.css('font-size',$('input').css('font-size'))
	// 	.insertBefore($(this));

	// 	$(this).hide().keyup(function(e)
	// 	{
	// 		var code = e.which; // recommended to use e.which, it's normalized across browsers
	// 		if ( code==27 ) {
	// 			e.preventDefault();
	// 			$(this).hide();
	// 			$("span[data-id='"+$(this).attr('id')+"']").show();
	// 		}
	// 	});
	// });

	// input number
	$("input[type=number]").each(function()
	{
		$(this).css('width', 24+(4*$(this).data('nod')) ); // number of digit
	});

	// update UI
	audioInit();
	playerInit();

	// load
	loadSettings();
	log();
});


/// SETTINGS ///////////////////////////////////////////////////////////////////////////////////////////////////////////

function loadSettings()
{
	$.ajax(
	{
		url: CGI,
		data: { do:"config",action:"load" },
		type: "POST",
		dataType: "json",
		timeout: AJAX_TIMEOUT_DELAY
	})
	.done(function(data)
	{

		var $elem   = $("#settings");
		var $alert  = $("#settings .alert");
		var $submit = $(".submit");

		if ( data.code ) $alert.addClass("error").text( "error: "+data.message ).show();

		if ( data.STATIC_ADDRESS ) $('#hostip').val( data.STATIC_ADDRESS );
		if ( data.HOSTNAME ) $('#hostname').val( data.HOSTNAME + ".local" );

		for( var p in data ) {
			switch(p) {

				// checkbox
				case 'WIFI_ENABLE' :
				case 'WIFI_SSID_IS_HIDDEN' :
				case 'WIFI_KEY_IS_HEXADECIMAL' :
				case 'REMOTE_ACCESS' :
				case 'CIRRUS_HEADPHONE' :
				case 'CIRRUS_LINE_OUT' :
				case 'CIRRUS_SPEAKER' :
				case 'CIRRUS_SPDIF_OUT' :
				case 'AUDIO_EQUALIZER' :
				case 'AUDIO_COMPRESSOR' :
				case 'AUDIO_LIMITER':
					$elem.find("input[name='"+p+"']").prop('checked', data[p]=='yes');
					break;

				// radio
				case 'WIFI_SECURITY' :
				case 'ONBOARD_AUDIO_OUTPUT' :
					$("input[name='"+p+"'][value='"+data[p]+"']").prop('checked', true).trigger('click');
					break;

				// sound card settings
				case 'SOUNDCARDS' :
					var i = 0;
					$.each( data[p], function(key,value) {
						soundcardEnable(value);
						$("select[name='SOUNDCARD']").append($("<option></option>")
							.attr("value",key)
							.text(soundcardName(value))
							.prop("selected",parseInt(data.SOUNDCARD)==key?"selected":""));
						i++;
					});
					$("select[name='SOUNDCARD']").prop('disabled', i<2);
					break;

				// scpecific
				case 'PLAYLIST' :
					playlistInit(data[p]);
					break;

				// misc
				default:
					$elem.find("input[name='"+p+"']").val( data[p] );
					break;
			}
		}

		$("input[type='range']").trigger('input');
		$("input[type='checkbox'].plugin:checked").trigger('change');

		$elem.find(":input")
		.keyup(function(){  $submit.show(); })
		.change(function(){ $submit.show(); });

		$alert.fadeOut( data.code ? FADEOUT_ERR_DELAY : FADEOUT_OK_DELAY );

	});
}

function saveSettings()
{
	var $elem   = $("#settings");
	var $alert  = $("#settings .alert");
	var $submit = $("#settings .submit");
	var address = $elem.find( "input[name='STATIC_ADDRESS']" ).val();
	var host    = $elem.find( "input[name='HOSTNAME']" ).val();

	// validate
	if ( $.trim(address) && !address.match(/^(\d{1,3}\.){3}\d{1,3}$/) ) {
		dialog( "INVALID STATIC ADDRESS", {
			message: "Please match the requested format (e.g. 192.168.0.2)"
		});
		return;
	}
	if ( !$.trim(host) ) {
		dialog("hostname could not be empty");
		return;
	}
	if ( !host.match(/^[a-zA-Z]+[\w\-]+\w+$/) ) {
		dialog( "INVALID HOSTNAME", {
			message: "Please match the requested format.\nRead the documentation for more information."
		});
		return;
	}


	// send data
	$submit.hide();
	$alert.removeClass("error").text("saving…").show();

	playlistSave();
	var data = { do:'config', action:'save' };
	$elem.find("input,select").each(function()
	{
		var type = $(this).attr("type");
		var name = $(this).attr("name");

		if ( name!=undefined ) switch( type ) {
			case 'checkbox' : data[name] = $(this).is(':checked') ? 'yes' : 'no'; break;
			case 'radio' : if ($(this).is(':checked')) data[name] = $(this).val(); break;
			default: data[name] = $(this).val(); break;
		}
	});

	$.post( CGI, data, function(data)
	{
		if ( data.code==200 ) {
			$alert.text(data.message).fadeOut( FADEOUT_OK_DELAY );

			var ask = host+".local"!=$('#hostname').val();
			$('#hostname').val(host+".local");

			if ( ask ) dialog(
				"THE HOSTNAME HAS CHANGED", {
				message: "Your new hostname will be available the next boot.\nDo you want to reboot now ?",
				action: function()
				{
					$.post( CGI, {do:'hostname',name:host}, function(response)
					{
						if ( !response || response.result.match(/reboot/i) ) reboot(true);
						else dialog( response.result );
					}
					, "json");
				},
				cancel: updateSettings
			});
			else updateSettings();

			netinfo();

		}
		else $alert.addClass("error").text("error: "+data.message).show().fadeOut(FADEOUT_ERR_DELAY);

	}, "json");
}

function updateSettings()
{

	var $elem = $("#settings");
	var addr  = $elem.find( "input[name='STATIC_ADDRESS']" ).val();

	var ask = addr!=$('#hostip').val();
	$('#hostip').val(addr);


	if ( ask ) dialog(
		"THE NETWORK ACCES WILL CHANGED", {
		message: "Do you want to update your network configuration now and reload this page ?",
		action: function()
		{
			dialog( "<a href='http://"+window.location+"'>updating</a> … wait", { class:'wait', action:false } );
			$.ajax({
				type: "POST",
				url: CGI,
				data: { do:"update" },
				timeout: 7000
			})
			.done( function(){ window.location.reload(); } )
			.fail( function()
			{
				var timer = setInterval(
					function()
					{
						$.ajax({ url: window.location, type: "HEAD", crossDomain: true })
						.done( function(result) { window.location.reload(); });
					},
					AJAX_TIMEOUT_DELAY
				);
			});
		}
	});
}






/// SERVICE ////////////////////////////////////////////////////////////////////////////////////////////////////////////

var log_t = null;
function log()
{
	$("#service .alert").text("refresh status…").removeClass("error");

	$.ajax({
		type: "POST",
		url: CGI,
		data: { do:"log" },
		timeout: AJAX_TIMEOUT_DELAY,
		dataType: "json"
	})
	.done(function(data)
	{
		$("#dialog").fadeOut();
		$("#service .alert").text(data.message).toggleClass("error",!data.active);
		$("#service .log").text(data.log);
		$("#service .info").text(data.info);
	})
	.fail( function()
	{
		dialog(
			'<a href="./">Your Raspberry PI</a> is not responding',{
			message : 'Please check your network connection, '+
			'cables, devices, (etc.), and wait a moment '+
			'before rebooting.',
			action: false
		});
	});
}

function startlogs()
{
	if ( !log_t ) log_t = setInterval( log, FETCH_STATUS_INTERVAL );
}

function stoplogs()
{
	if ( log_t ) {
		clearInterval(log_t);
		log_t = null;
	}
}

// stream control
function servicectl( action )
{
	$("#service .alert").removeClass("error").text(action+" wait…");
	$.post( CGI, {do:"service",ctl:action}, log );
}







/// SYSTEM /////////////////////////////////////////////////////////////////////////////////////////////////////////////

function sysclear()
{
	$('#system .info').html( '<img src="img/loading.png" width="24" class="loading"> system information …' );
}

function sysinfo()
{
	$.post( CGI, {do:"sysinfo"}, function(content){
		$('#system .info').text(content);
	}, "text" );
}

function netinfo()
{
	
	$("#settings .alert").text("fecth info…").show();
	$.post( CGI, {do:"netinfo"}, function(data)
	{
		$("#settings .alert").fadeOut( FADEOUT_OK_DELAY );

		if ( data.eth  ) $("#settings .info.eth").text(data.eth);
		if ( data.wifi ) $("#settings .info.wifi").text(data.wifi);
		if ( $("#settings").find("input[name='REMOTE_ACCESS']").is(':checked') ) {
			if ( data.remote.port.message ) $("#settings .info.remote").text( data.remote.port.message );
			else $("#settings .info.remote").text(
				"external interface address: http://"+data.remote.ip+":"+data.remote.port.http+"\n"+
				"external ssh access: ssh -p "+data.remote.port.ssh+" root@"+data.remote.ip
			);
		}
		else $("#settings .info.remote").text("");
		
	}, "json");
}

// args: option, param1, param2, ...
function netctl()
{

	var args = [].slice.call(arguments);
	var emsg = "#"+$( args.shift() ).closest("div.panel").attr('id')+" .alert";


	if ( args[1]=='scan' ) $(emsg).removeClass("error").text('Looking for Networks…').show();
	exec('sudo '+NETWORK_CGI+' --'+args.join(' '), function()
	{
		$(emsg).fadeOut( FADEOUT_OK_DELAY );
	});
}

function exec( cmd, callback )
{
	$.post( CGI, {do:"exec","cmd":cmd}, function( data )
	{
		overlay('#console');
		$("#console .title").text( data.cmd );

		var pre = $("#console").find("pre");
		pre.text( data.result );
		if ( pre.height()>$(window).height()-120 ) pre.css( "height", $(window).height()-120 );
		else pre.css( "height", "auto" );
		pre.hide().slideDown();

	}, "json")
	.done( callback );
}

function shutdown()
{
	dialog( "shutdown…", { class:'wait', action:false } );
	
	$.ajax({
		type: "POST",
		url: CGI,
		data: { do:"shutdown" },
		timeout: AJAX_TIMEOUT_DELAY,
		dataType: "text"
	})
	.done(function(msg)
	{
		$("#dialog").fadeOut();
		$("#system .alert").addClass("error").text(msg).show();
	})
	.fail( function()
	{
		var host = $('#hostname').val();
		dialog( "<a href='http://"+host+"/'>"+host+"</a> is OFF", { class:'halt', action:false } );
		stoplogs();
	});
}

function reboot()
{

	var host = $('#hostname').val();
	dialog( "<a href='http://"+host+"/'>rebooting</a> … wait", { class:'wait', action:false } );
	stoplogs();
	
	var xhr = null;
	var timer = setInterval( function(){
		if ( xhr ) xhr.abort();
		xhr = $.ajax({ url: window.location, type: "HEAD", crossDomain: true })
		.done( function() { window.location = "http://"+host+"/"; });
	},
	AJAX_PING_FREQ);

	// fake reboot or already sent by another method
	if ( arguments.length ) return;

	$.post( CGI, {do:"reboot"}, function(msg)
	{
		if ( timer ) {
			clearInterval(timer);
			timer = null;
		}
		$("#dialog").fadeOut();
		$("#system .alert").addClass("error").text(msg).show();

	}, "text");
}




/// INTERFACE //////////////////////////////////////////////////////////////////////////////////////////////////////////

// open message in dialog window
// optionnal args : { class:*, message:*, action:*, cancel:* }
function dialog( title, args )
{
	if ( !title ) return;

	if ( typeof(args)!='object' ) args = {};
	if ( !args.class ) args.class = "warning";

	var content = "<p class='title'>" + title + "</p>";
	if ( args.message ) content += "<p class='message'>"+ ( args.message.replace(/\n/g,"<br>") ) +"</p>";

	// alert
	if ( typeof(args.action)=='undefined' ) {
		content += "<p class='action'>";
		content += "<input type='button' value='Ok' onclick=\"$('#dialog').fadeOut();\">";
		content += "</p>";
	}

	// confirm
	if ( typeof(args.action)=='function' ) {
		content += "<p class='action'>";
		content += "<input type='button' value='Cancel'>";
		content += "<input type='button' value='Ok'>";
		content += "</p>";
	}

	$("#dialog").html( "<div class='"+args.class +"'>"+content+"</div>" ).stop().css('opacity',1).show();
	$("#dialog").find("input[type='button']").each(function(i)
	{
		if ( !i ) $(this).focus().click(function(){$('#dialog').fadeOut();}).click(args.cancel);
		else $(this).click(function(){$('#dialog').fadeOut();}).click(args.action);
		$('#dialog > div').css("padding-bottom", 0);
	});
}

function toggle( elem, target )
{
	var m = $( elem ).text().match(/View|Hide/);
	var t = $( elem ).text().replace(/View|Hide/,'');
	$( target ).toggle();
	if ( m ) $( elem ).text( $(target).is(":hidden") ? "View"+t : "Hide"+t );
}

function overlay( target )
{
	$(target).addClass('overlay');
	$(target).show().click( function(e) { if ( e.target==this ) $(this).fadeOut(); });
}




/// AUDIO //////////////////////////////////////////////////////////////////////////////////////////////////////////////

function audioInit()
{
	// select dynamically
	$("select[name='SOUNDCARD']").on('change',function()
	{
		audioUpdate("card", $(this).val() );
	});

	// sent change dynamically
	$('input.dyn').on('change',function()
	{
		var keys = $(this).attr('class').replace(/(\s+)?dyn/,'').replace(/(\s+)?sign/,'');
		if ( $(this).hasClass('percent') ) audioUpdate( "master", $(this).val()+'%' );
		else audioUpdate( $(this).closest('div').attr('id'), keys, $(this).val() );
	});

	// ad sign to all range type gain
	$("input[type='range'].gain").addClass('sign');

	// add slider output value
	// value to DB  = ( value-(Math.abs(min)/step) )*step
	// DB to value  = ( value+Math.abs(min) ) / step
	$("input[type='range']").each(function(){
		//$(this).parent().attr( 'class', $(this).attr('class') );
		$(this).after( '<span id="'+$(this).attr('name')+'" class="range"></span>' );
		$(this).on('input',function(){
			var val = parseFloat( $(this).val() );
			var sign = unit = '';

			if ( $(this).hasClass('sign') )    sign = val==0 ? ' ' : (val<0 ? '-' : '+');
			if ( $(this).hasClass('gain') )    unit = ' dB';
			if ( $(this).hasClass('percent') ) unit = ' %';
			if ( $(this).hasClass('sec') )     unit = ' sec';

			$('#'+$(this).attr('name')).text( sign+' '+Math.abs($(this).val()).toFixed(2)+unit );
		});
	});

	// plugion
	$("input[type='checkbox'].plugin").on('change',function()
	{
		$('#'+$(this).data('target')).toggle();
	});
}

// args: option, param1, param2, ...
function audioUpdate()
{
	var args = [].slice.call(arguments);
	$.post( CGI, {do:"exec",quiet:true,cmd:'sudo '+AUDIO_CGI+' --'+args.join(' ')} );
}

// args: option, param1, param2, ...
function audioExec()
{
	var args = [].slice.call(arguments);
	exec('sudo '+AUDIO_CGI+' --'+args.join(' '));
}

function audioTest(button)
{
	var data = {
		do: "audiotest",
		action: button.value,
		samplerate: $("select[name=AUDIO_TEST_SAMPLERATE]").val(),
		channel: $("input[name=AUDIO_TEST_CHANNEL]").val(),
		type: $("input[name=AUDIO_TEST_TYPE]:checked").val()
	};

	$.ajax({
		url: CGI,
		data: data,
		type: "POST",
		dataType: "text",
		timeout: 3000
	})
	.done(function(msg)
	{
		if ( msg.match(/invalid argument/i) ) {
			dialog("Invalid argument",{message:msg});
			button.value="start";
		}
		$("#audio-status_resp").removeClass("warning").fadeOut( FADEOUT_OK_DELAY );
	});

	if ( button.value=="stop" ) {
		$("#audio-status_resp").removeClass("warning").fadeOut( FADEOUT_OK_DELAY );
		button.value = "start";
	}
	else {
		$("#audio-status_resp").html("running test <span class=\"running\"></span>").addClass("warning").show();
		button.value = "stop";
	}
}

function soundcardEnable(name)
{
	if ( name=="snd_rpi_wsp"  	  		) $(".cirrus").toggle();
	if ( name=="RPi-Cirrus"  	  		) $(".cirrus").toggle();
	if ( name=="Blue Icicle"  	  		) $(".blueicicle").toggle();
	if ( name=="bcm2835 ALSA" 	  		) $(".onboard").toggle();
	//if ( name=="USB Audio Device" 		) $(".usbaudiodev").toggle();
	if ( name=="seeed-2mic-voicecard"	) $(".respeaker").toggle();
	if ( name=="seeed-4mic-voicecard"	) $(".respeaker").toggle();
	if ( name=="seeed-8mic-voicecard"	) $(".respeaker").toggle();
	if ( name=="snd_rpi_hifiberry_dac"	) $(".hifiBerry").toggle();
}

function soundcardName(name)
{
	if ( name=="bcm2835 ALSA" 			) return "onboard audio outputs";
	if ( name=="snd_rpi_wsp"  			) return "Cirrus Logic/Wolfson audio card";
	if ( name=="RPi-Cirrus"  			) return "Cirrus Logic/Wolfson audio card";
	if ( name=="seeed-2mic-voicecard"	) return "ReSpeaker 2-Mics";
	if ( name=="seeed-4mic-voicecard"	) return "ReSpeaker 4-Mics";
	if ( name=="seeed-8mic-voicecard"	) return "ReSpeaker 6-Mics";
	if ( name=="snd_rpi_hifiberry_dac"	) return "HifiBerry audio card";

	return name;
}







/// PLAYER /////////////////////////////////////////////////////////////////////////////////////////////////////////////


var player =
{

	items: [],
	index: -1
}

function playerInit()
{
	$("<AUDIO/>").attr('control',"control").insertAfter('#playlist a');
	$('#playlist a').css('display','block').click(function()
	{
		playlistAddItem('');
	});
}

function playlistInit( data )
{
	if ( !data || data==undefined ) data = "";
	else data = data.replace(/^;+|;+$/g,'')+';';


	var items = data.split(/[;]+/);
	items.forEach(playlistAddItem);
}

function playlistAddItem( url )
{
	var idx = player.items.length;
	if ( url && url!=undefined ) player.items.push(url);

	$('<INPUT/>')
	.addClass("playlist-item editable")
	.attr('id','playlist-item-'+idx)
	.attr('placeholder','URL or file path').val( url )
	.insertBefore($('#playlist a'));

	$('<IMG/>')
	.addClass('playlist-item')
	.attr('src','img/play.png')
	.attr('data-index',idx)
	.click(function()
	{
		$('#playlist img').attr('src','img/play.png');

		var idx  = $(this).data('index');
		var url  = $('#playlist-item-'+idx).val().trim();
		var elem = $('audio').get(0);


		if ( url!=undefined && player.index!=idx ) {
			$(this).attr('src','img/pause.png');
			player.index = idx;
			elem.src = url.replace(/\/home\/locusonus/,'');
			elem.play();
		}
		else {
			player.index = -1;
			elem.pause();
		}

	})
	.insertBefore($('#playlist a'));

}

function playlistSave()
{
	var value = [];
	$('#playlist input.playlist-item').each(function()
	{
		var v = $(this).val().trim();
		if (v) value.push(v);
	});
	
	$("input[name='PLAYLIST']").val( value.join(';') );
}

/*function command()
{
	$("#player").find("input.editable").keyup(function(e)
	{
		var code = e.which; // recommended to use e.which, it's normalized across browsers
		if ( code==13 ) {
			e.preventDefault();
			savePlayerConfig();
		}
		// if ( code==27 ) {
		// 	e.preventDefault();
		// 	$(this).hide();
		// 	$("#player").find("span[data-id='"+$(this).attr('id')+"']").show();
		// }
	});
}*/






