<?php

	/**
	 * (./) process.php
	 * (by) cousot stéphane @ http://www.ubaa.net/
	 * (cc) some right reserved
	 *
	 * User request's dispatcher, backend process.
	 * version 0.1
	 * 
	 *
	 * -----------------------------------------------------------------------------------------------------------------
	 *
	 *  This file is part of the Locus Sonus Streambox Player project. http://locusonus.org/streambox/
	 *
	 *  THIS CODE ARE RELEASED UNDER A CREATIVE COMMONS ATTRIBUTION 4.0 INTERNATIONAL LICENSE
	 *  @ http://creativecommons.org/licenses/by/4.0/
	 *
	 * -----------------------------------------------------------------------------------------------------------------
	 */



	// config file(s)
	define( "USER_CONFIG",    "/boot/settings.txt" );
	define( "DEFAULT_CONFIG", "/home/locusonus/www/conf/default" );
	define( "DEFAULT_SCRIPT", "/home/locusonus/www/conf/script" );

	// script(s)
	define( "NETWORK_SCRIPT", "/home/locusonus/bin/network.sh" );
	define( "AUDIO_SCRIPT", "/home/locusonus/bin/audio.sh" );

	// systemd servive(s)
	define( "SERVICE", "streambox-player" );
	define( "SERVICE_STATUSLED", SERVICE."-statusled" );



	# handle reboot 
	if ( $_SERVER['REQUEST_METHOD']==='HEAD' ) die;

	// handle redirections / requests
	if ( $_SERVER['REDIRECT_STATUS']=="404" ) {
		$what = trim( $_SERVER['REQUEST_URI'], '/' );
		if ( $what=="start" || $what=="stop" || $what=="restart" ) do_service( $what );
		else header("${_SERVER['SERVER_PROTOCOL']} 404 Not Found");
		die;
	}

	# accept only POST from host referer
	# or respond with a 403 status code
	if ( !$_POST || !preg_match("/^http:\/\/${_SERVER['HTTP_HOST']}\//", $_SERVER["HTTP_REFERER"]) )
	{
		header("${_SERVER['SERVER_PROTOCOL']} 403 Forbidden");
		die;
	}


	# dispatch
	header('Content-Type: text/plain; charset=utf-8');
	switch( $_POST['do'] )
	{
		case 'log'		 : do_log(); break;
		case 'config'	 : do_config($_POST['action']); break;
		case 'service'	 : do_service($_POST['ctl']); break;
		case 'hostname'	 : sethostname($_POST['name']); break;
		case 'netinfo'	 : do_netinfo(); break;
		case 'exec'		 : do_exec($_POST['cmd'],$_POST['quiet']); break;
		case 'sysinfo'	 : do_sysinfo(); break;
		case 'update'	 : system( "sudo ".NETWORK_SCRIPT." --update" ); break;
		case 'reboot'	 : do_reboot(); break;
		case 'shutdown'	 : do_shutdown(); break;
		case 'audiotest' : do_audiotest($_POST['action'],$_POST['type'],$_POST['channel'],$_POST['samplerate']); break;
		
		default: break;
	}


	# ------------------------------------------------------------------------------------------------------------------
	# 	CONFIG
	# ------------------------------------------------------------------------------------------------------------------

	function do_config( $action )
	{
		if ( $action=="load" ) load_config();
		if ( $action=="save" ) save_config();
	}

	function load_config()
	{

		if ( !file_exists(USER_CONFIG) ) die(json_encode(["code"=>404,"message"=>"No such file or directory"]));
		if ( !is_readable(USER_CONFIG) ) die(json_encode(["code"=>403,"message"=>"Permission denied"]));

		$data = @file_get_contents(USER_CONFIG) or die(json_encode(["code"=>204,"message"=>"Empty settings"]));

		preg_match_all( '/(^|#)(\w.*)="(.*)"/m', $data, $matched );
		$vars = array_combine( $matched[2], $matched[3] );
		$vars["HOSTNAME"] = $vars["HOSTNAME"] ?: "streambox-player";

		preg_match_all('/card (\d+).*\[(.*)\],/', shell_exec('sudo aplay -l'), $cards );
		$vars["SOUNDCARDS"] = @array_combine( $cards[1], $cards[2] );

		print json_encode( $vars );
	}

	function save_config()
	{
		$temp = !is_writable( USER_CONFIG );
		$file = !$temp ? USER_CONFIG : tempnam(sys_get_temp_dir(),'streambox-'); 

		if ( !is_readable(DEFAULT_CONFIG) ) die(json_encode(["code"=>403,"message"=>"Permission denied"]));
		
		$data = @file_get_contents(DEFAULT_CONFIG);
		foreach( $_POST as $key=>$value ) {
			$data = preg_replace("/(^|#)$key(\s+)?=.*$/m", "$key=\"$value\"", $data );
		}

		enable_service( SERVICE_STATUSLED, $_POST['STATUSLED']=="yes" );
		remote_update( $_POST );

		$handle = fopen( $file, "w" );
		if ( $handle ) {
			fwrite( $handle, $data );
			fclose( $handle );
		}

		if ( $temp ) system( "sudo /usr/bin/cp $file ".USER_CONFIG );
		system( "sudo ".AUDIO_SCRIPT." --setup quiet" );

		print json_encode(["code"=>200,"message"=>"saved"]);
	}

	function parse_config()
	{
		$config = @file_get_contents(USER_CONFIG);
		$config = preg_replace( "/#.*/", "", $config );
		
		return parse_ini_string($config);
	}




	# ------------------------------------------------------------------------------------------------------------------
	#	SERVICE
	# ------------------------------------------------------------------------------------------------------------------

	function do_log()
	{

		// TODO: fix timestamp
		// since the lastest activation
		//$jrl = shell_exec("sudo journalctl --no-pager -u ".SERVICE." --since \"$(echo $(sudo systemctl show -p ActiveEnterTimestamp ".SERVICE.") | awk '{print $2 \" \" $3}')\"");
		// since the lastest boot
		$jrl = shell_exec("sudo journalctl --no-pager -u ".SERVICE." -b");
		
		// since last starting
		$matches = preg_grep('/.*Starting/', explode("\n", $jrl));
		$jrl = substr($jrl,strpos($jrl,end($matches)));

		//$log = shell_exec("sudo systemctl status -n 0 ".SERVICE).substr($jrl,strpos($jrl,"\n")); // remove first line
		$log = shell_exec("sudo systemctl status -n 0 ".SERVICE)."\n".$jrl;
		preg_match("/.*Active:(.*)/i", $log, $matched );


		// check for error
		if ( $log ) {
			$log = preg_replace("/.*Terminal type `unknown' is not defined\.\\n/", "", $log);
			if ( preg_match("/failed to write buffer data/i", $log) ) {
				$matched[1] = "internal error";
				do_service( "stop" );
			}
			if ( preg_match("/no soundcards found/i", $log) ) {
				$matched[1] = "no soundcards found";
			}
			if ( preg_match("/no command lines found/i", $log) ) {
				$matched[1] = "command not found";
			}
		}
		else $matched[1] = $log = "Service not found: ".SERVICE;



		// get system load average
		$cpu = array( "tot"=>0, "bin"=>array() );
		$cmd = service_command();
		if ( !$cmd['error'] ) foreach( $cmd as $key=>$value) {
			$percent = trim(shell_exec( "ps -C $value -o %cpu | sed -n 2p" ))?:0;
			$cpu['tot'] += $percent;
			array_push( $cpu['bin'], "$value: $percent%" );
		}

		// get network traffic
		$net   = nettraffic( "eth0" );
		$temp  = shell_exec( "/opt/vc/bin/vcgencmd measure_temp" );
		$volts = shell_exec( "/opt/vc/bin/vcgencmd measure_volts core" ); 

		$info = sprintf(
			"average CPU usage ... %s".
			"\nprocess CPU usage ... %s".
			"\ntotals upload ....... %s".
			"\ntotals download ..... %s".
			"\ntemperatures ........ %s°c".
			"\ncore voltages ....... %sv".
			"\n",
			"${cpu['tot']}%", implode(", ",$cpu['bin']),
			$net['tx'], $net['rx'],
			preg_replace("/[^0-9\.]/","",$temp),
			preg_replace("/[^0-9\.]/","",$volts)
		);


		print json_encode([
			"message" => $matched[1],
			"active"  => stripos($matched[1],"running")!==false,
			"log" 	  => $log,
			"info"    => $info
		]);
	}

	function service_command()
	{
		$conf = parse_config();
		$cmd  = preg_split( "/\s+\|\s+/", $conf['COMMAND_LINE'], -1, PREG_SPLIT_NO_EMPTY );
		$bin  = array();

		foreach( $cmd as $key=>$value ) array_push( $bin, current(explode(' ',$value)) );
		return $bin ? $bin : ["error"=>404,"message"=>"Empty commands"];
	}





	# ------------------------------------------------------------------------------------------------------------------
	#	NETWORK
	# ------------------------------------------------------------------------------------------------------------------

	function do_netinfo()
	{
		print json_encode([
			"eth"    => netstatus("eth0"),
			"wifi"   => netstatus("wlan0"),
			"remote" => array(
				"ip" => @file_get_contents("http://pi.ubaa.net/ip"),
				"port" => remote_port()
			)
		]);
	}

	function netaddress()
	{
		return array(
			"lan"  => trim(shell_exec("hostname -i")),
			"wan"  => @file_get_contents("http://pi.ubaa.net/ip")
		);
	}

	function sethostname( $name )
	{
		do_exec("sudo ".NETWORK_SCRIPT." --hostname $name");
	}

	function netstatus( $link, $list=false )
	{
		$status  = shell_exec("networkctl status $link");
		
		preg_match("/^\s+state: (.*)/im", $status, $state );
		preg_match("/^\s+address: (.*)/im", $status, $address );
		preg_match("/\sdns: (.*)\s+(.*)/im", $status, $dns );
		preg_match("/^\s+model: (.*)/im", $status, $model );

		$dns = $dns[1][0] ? $dns[2] ? $dns[1]."/".$dns[2] : $dns[1] : "";

		if ( !$list ) return sprintf(
			"state: %s\naddress: %s%s%s",
			$state[1], $address[1], $dns ? "\nDNS: ".$dns:"", $traffic?"\n$traffic":""
		);

		return array(
			"model" => $model[1],
			"state" => $state[1],
			"address" => $address[1],
			"DNS" => $dns
		);
	}

	function nettraffic( $link )
	{
		$ifconfig = shell_exec("ifconfig $link");

		preg_match("/^\s+RX packets.*\((.*)\)/im", $ifconfig, $rx );
		preg_match("/^\s+TX packets.*\((.*)\)/im", $ifconfig, $tx );

		return $rx||$tx ? array("tx"=>$tx[1],"rx"=>$rx[1]) : false;
	}

	function remote_update( $data )
	{
		$ini = parse_config();
		if ( $ini &&
			 $ini['REMOTE_ACCESS']!=$data['REMOTE_ACCESS'] || 
			($ini['REMOTE_ACCESS']=="yes" && $ini['REMOTE_ACCESS_HTTP_PORT']!=$data['REMOTE_ACCESS_HTTP_PORT']) || 
			($ini['REMOTE_ACCESS']=="yes" && $ini['REMOTE_ACCESS_SSH_PORT'] !=$data['REMOTE_ACCESS_SSH_PORT'])
		){
			$todo = $data['REMOTE_ACCESS']=="no" ? "stop" : "start";
			shell_exec( "sudo ".NETWORK_SCRIPT." --remote $todo" );
		}
	}

	function remote_port()
	{
		$data = parse_config();

		if ( $data['REMOTE_ACCESS']=="yes" ) {
			$status = shell_exec("sudo iptables -t nat -n -L");
			preg_match( "/.*tcp dpt:(\d+) redir ports 80/", $status, $HTTP );
			preg_match( "/.*tcp dpt:(\d+) redir ports 22/", $status, $SSH );

			if ( $HTTP || $SSH ) return array(
				"http" => $HTTP[1],
				"ssh"  => $SSH[1],
			);
			else {
				$message = shell_exec("sudo ".NETWORK_SCRIPT." --remote scan 2>&1");
				if ( !$message ) $message = "Unable to found remote access configuration !";
				return ["code"=>204,"message"=>$message];
			}
		}

		return false;
	}





	# ------------------------------------------------------------------------------------------------------------------
	#	AUDIO
	# ------------------------------------------------------------------------------------------------------------------

	function do_audiotest( $action, $type="pink", $channel=2, $samplerate=48000 )
	{
		if ( $action=="stop" ) system("sudo ".AUDIO_SCRIPT." --test $action");
		else system("sudo speaker-test -l0 -t $type -c $channel -r $samplerate");
	}


	# ------------------------------------------------------------------------------------------------------------------
	#	SYSTEM
	# ------------------------------------------------------------------------------------------------------------------

	// function do_loadfile( $file )
	// {
	// 	if ( !file_exists($file) ) die( "No such file or directory: $file" );
	// 	if ( !is_readable($file) ) die( "Permission denied: $file" );

	// 	print @file_get_contents($file);
	// }

	function do_exec( $cmd, $quiet=false )
	{
		$cmd = escapeshellcmd($cmd);
		if ( $quiet ) exec( "$cmd &" );
		else print json_encode([
			"cmd" => $cmd,
			"result" => shell_exec( $cmd.' 2>&1' )
		]);
	}

	function do_reboot()
	{
		do_service("stop");
		system("sudo /usr/bin/systemctl reboot -i");
		sleep(10);
		print "reboot failed !";
	}

	function do_shutdown()
	{
		do_service("stop");
		system("sudo /usr/bin/systemctl poweroff -i");
		sleep(10);
		print "shutdown failed !";
	}

	function do_service( $action, $unit=SERVICE )
	{
		system( "sudo systemctl $action $unit" );
	}

	function enable_service( $unit, $enable )
	{
		$status = shell_exec("sudo systemctl is-active $unit");
		if ( $enable && trim($status)=="unknown" ) system("sudo systemctl start $unit" );
		if ( !$enable && trim($status)!="unknown" ) system("sudo systemctl stop $unit" );

		$status = shell_exec("sudo systemctl is-enabled $unit");
		if ( $enable && trim($status)!="enabled" ) system("sudo systemctl enable $unit" );
		if ( !$enable && trim($status)!="disabled" ) system("sudo systemctl disable $unit" );
	
	}

	function do_sysinfo()
	{

		$name = shell_exec( "grep -E \"^#\s+name:\" ".DEFAULT_CONFIG." | cut -d: -f2" );
		$version = shell_exec( "grep -E \"^#\s+version:\" ".DEFAULT_CONFIG." | cut -d: -f2" );
		$net = netaddress();

		$cpu = shell_exec("cat /proc/cpuinfo");
		preg_match("/^model name\s+: (.*)/im", $cpu, $proc);
		preg_match("/^(ARMv\d+)/i", $proc[1], $arch);

		// http://elinux.org/RPi_HardwareHistory#Board_Revision_History
		$model = shell_exec("cat /proc/cpuinfo | grep 'Revision' | awk '{print $3}' | sed 's/^1000//'");
		switch( trim($model) ) {
			case "Beta"   : $model = "Model B (beta)"; break;
			case "0002"   : 
			case "0003"   : $model = "Model B Revision 1.0"; break;
			case "0004"   : 
			case "0005"   : 
			case "0006"   : $model = "Model B Revision 2.0"; break;
			case "0007"   : 
			case "0008"   : 
			case "0009"   : $model = "Model A Revision 2.0"; break;
			case "000d"   : 
			case "000e"   : 
			case "000f"   : $model = "Model B Revision 2.0"; break;
			case "0010"   : $model = "Model B+ Revision 1.0"; break;
			case "0011"   :
			case "0014"   : $model = "Compute Module 1 Revision 1.0"; break;
			case "0012"   : 
			case "0015"	  : $model = "Model A+ Revision 1.1"; break;
			case "0013"   : $model = "Model B+ Revision 1.2"; break;
			case "a01040" : $model = "2 Model B Revision 1.0"; break;
			case "a01041" : 
			case "a21041" : $model = "2 Model B Revision 1.1"; break;
			case "a22042" : $model = "2 Model B Revision 1.2"; break;
			case "900021" : $model = "Model A+ Revision 1.1"; break;
			case "900032" : $model = "Model B+ Revision 1.1"; break;
			case "900092" : $model = "Model Zero Revision 1.2"; break;
			case "900093" : 
			case "920093" : $model = "Model Zero Revision 1.3"; break;
			case "9000c1" : $model = "Model Zero W Revision 1.1"; break;
			case "a02082" : 
			case "a22082" : 
			case "a32082" : $model = "3 Model B Revision 1.2"; break;
			case "a020a0" : $model = "Compute Module 3 Revision 1.0"; break;
			case "a020d3" : $model = "3 Model B+ Revision 1.3"; break;
			default: $model = "unknown (model revision: $model)"; break;
		}


		$df = shell_exec("df -H -t ext4 --output=size,used,pcent,avail | sed -n 2p");
		$disk = preg_split("/ /", trim($df), -1, PREG_SPLIT_NO_EMPTY);


		$mem = shell_exec("/opt/vc/bin/vcgencmd get_mem arm");
		preg_match("/arm=(\d+M)/Ui", $mem, $memory['arm']);

		$mem = shell_exec("/opt/vc/bin/vcgencmd get_mem gpu");
		preg_match("/gpu=(\d+M)/Ui", $mem, $memory['gpu']);


		printf(
			"%s %s (%s)".
			"\nnetwork node name ............. %s".
			"\nlocal address ................. %s".
			"\npublic address ................ %s".
			"\noperating system .............. %s".
			"\nkernel ........................ %s".
			"\nprocessor ..................... %s".
			"\nboard version ................. %s".
			"\ndisk usage .................... %s (size), %s (%s used), %s (avail)".
			"\nmemory allocated to the CPU ... %s".
			"\nmemory allocated to the GPU ... %s".
			"\n",
			str_pad(trim($name)." ",31,'.'), trim($version), strtolower($arch[1]),
			uname('n'), $net['lan'], $net['wan'],
			uname('o'), uname('srv'),
			$proc[1], $model,
			$disk[0], $disk[1], $disk[2], $disk[3],
			$memory['arm'][1], $memory['gpu'][1]
		);
	}

	function uname( $opt )
	{
		return trim( shell_exec("uname -$opt") );
	}

?>
